package org.tekkotsu.ui.rcp.actions;

import java.io.File;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.FileDialog;
import org.tekkotsu.ui.rcp.StoryboardView;
import org.tekkotsu.ui.util.Debugger;
import org.tekkotsu.ui.util.FilesystemUtil;

public class SaveTraceAction extends Action {
  private StoryboardView view;

  public SaveTraceAction(StoryboardView v)
  {
	  super("Save Trace As");
	  this.view = v;
  }
  
  private File queryFile() {
		FileDialog dialog = new FileDialog(view.viewer.getControl()
				.getShell(), SWT.SAVE);
		dialog.setFilterExtensions(new String[] { "*.tse" });
		dialog.setText("Save Trace File as"); //$NON-NLS-1$

		FilesystemUtil.setFileDialogStartPath(dialog);
		String path = dialog.open();
		FilesystemUtil.stickyPathSet(path);

		if (path != null && path.length() > 0) {
			if (!path.endsWith(".tse"))
				path = path + ".tse";
			return new File(path);
		}
		return null;
	}

	@Override
	public void run() {
		String filename = "";
		try {
			File file = queryFile();

			if (file == null) {
				// cancel button was pressed
				return;
			}

			filename = file.getName();
			view.viewer.setContent(view.content);
			view.viewer.updateAllInput(view.editor.getLayoutData());
			view.content.saveTrace(file);
		} catch (Exception e) {
			Debugger.printThrowable(e);
			String msg = "Cannot write trace file " + filename;
			MessageDialog.openError(view.viewer.getControl().getShell(),
					"Problem", msg);
		}
	}
}
