/*
 * Created on Nov 9, 2004
 */
package org.tekkotsu.ui.storyboard.panels;

import java.beans.PropertyChangeListener;

import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.Panel;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.swt.graphics.RGB;
import org.tekkotsu.ui.storyboard.ResourceRegistry;
import org.tekkotsu.ui.storyboard.objects.AbstractViewObject;
import org.tekkotsu.ui.storyboard.views.StoryboardViewer;
import org.tekkotsu.ui.util.Debugger;

public class TimeLine extends AbstractTimePanel implements PropertyChangeListener {
	public static final int X_POS  = 0;
	public static final int Y_POS  = 0;
	public static final int HEIGHT = 25;

	int cursor = 0;

	int timeLimit = 0;

	int maxValue = 0;

	Rectangle constraint;

	public final static String COLOR_BACKGROUND = "_timeline_bg";
	public final static String COLOR_EVENT = "timeline_event";
	public final static String COLOR_BORDER = "timeline_border";
	public final static String COLOR_MINOR_SECOND_TICK = "timeline_minor_second_tick";
	public final static String COLOR_MAJOR_SECOND_TICK = "timeline_major_second_tick";
	public final static String COLOR_TEXT = "timeline_text";
	public final static String COLOR_CURSOR = "timeline_cursor";

	@Override
	protected int getPreferredX() {
		return X_POS;
	}

	@Override
	protected int getPreferredY() {
		return Y_POS;
	}
	
	@Override
	protected int getPreferredH() {
		return HEIGHT;
	}
	
	public int getLimit() {
		return timeLimit;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.draw2d.Figure#paintFigure(org.eclipse.draw2d.Graphics)
	 */
	@Override
	protected void paintFigure(Graphics gc) {
		final Rectangle bounds = getBounds();
		final Rectangle viewbox = viewer.getVisibleViewport();

		final int BX = bounds.x;
		final int BY = bounds.y;
		final int BW = bounds.width;
		final int BH = bounds.height;
		final int VIEW_L = viewbox.x - 50;
		final int VIEW_R = VIEW_L + viewbox.width + 50;
		final int CHAR_WIDTH = gc.getFontMetrics().getAverageCharWidth();
		final int LANDMARK_SECOND_CAP = 5;

		// Draw the background
		gc.setBackgroundColor(colorReg.get(COLOR_BACKGROUND));
		gc.fillRectangle(bounds);

		/*
		 * draw tick every second and label landmark seconds
		 */
		int clearAfter = 0;
		for (int second = getTimeOfOffset(VIEW_L)/1000; ; ++second) {
			int x = getOffsetOfTime(second*1000);

			if (x < VIEW_L) {
				Debugger.printDebug("TIMELINE TOO LEFT");
			} else if (x > VIEW_R) {
				Debugger.printDebug("TIMELINE TOO RIGHT");
				break;
			}

			// draw seconds tick
			if (second % LANDMARK_SECOND_CAP == 0) {
				gc.setForegroundColor(colorReg.get(COLOR_MAJOR_SECOND_TICK));
			} else {
				gc.setForegroundColor(colorReg.get(COLOR_MINOR_SECOND_TICK));
			}
			
			gc.drawLine(BX + x, BY, BX + x, BY + 4);
			gc.drawLine(BX + x, BY + BH, BX + x, BY + BH - 5);

			if (second % LANDMARK_SECOND_CAP == 0) {
				//label seconds
				final String labelText = "" + second;
				final int LABEL_W = labelText.length() * CHAR_WIDTH;
				final int LABEL_X = x - LABEL_W / 2 - 1;
				
				if ((LABEL_X - clearAfter > 10) ) {
					gc.setForegroundColor(colorReg.get(COLOR_TEXT));
					gc.drawString(labelText, BX + LABEL_X, BY + 4);
				}
				clearAfter = LABEL_X + LABEL_W;
			}
		}

		gc.setForegroundColor(colorReg.get(COLOR_BORDER));
		// draw top border
		gc.drawLine(BX, BY, BX + BW, BY);
		// draw bottom border
		gc.drawLine(BX, BY + BH - 1, BX + BW, BY + BH - 1);
		
		// draw cursor
		gc.setXORMode(true);
		gc.setBackgroundColor(colorReg.get(COLOR_CURSOR));
		gc.fillRectangle(BX + getMarker().getOffset(getTimeCursor()) - halfgap, BY + 1,
				gap, BH - 2);
		gc.setXORMode(false);
	}

	/**
     *  
     */
	public TimeLine(Panel parent, StoryboardViewer viewer) {
		super(parent, viewer,new TimeMarker(xoffset,gap,timeScale));

		// initiate color registry
		colorReg = ResourceRegistry.getInstance().getColorRegistry();
		colorReg.put(COLOR_BACKGROUND, new RGB(235, 235, 228));
		colorReg.put(COLOR_EVENT, new RGB(150, 150, 150));
		colorReg.put(COLOR_BORDER, new RGB(0, 0, 0));
		colorReg.put(COLOR_MAJOR_SECOND_TICK, new RGB(0, 0, 0));
		colorReg.put(COLOR_MINOR_SECOND_TICK, new RGB(150, 150, 150));
		colorReg.put(COLOR_TEXT, new RGB(0, 0, 0));
		colorReg.put(COLOR_CURSOR, new RGB(0, 128, 128));
	}

	@Override
	public void update(AbstractViewObject child) {
		throw new UnsupportedOperationException();
	}
}